#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core import debug
from fenrirscreenreader.core.screenDriver import ScreenDriver as screenDriver


class driver(screenDriver):
    def __init__(self):
        screenDriver.__init__(self)
