#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

import builtins
import gettext
import locale

# the only function we really need to call here is gettext.install.
# Python3 has simplified this.
gettext.install("fenrir")

# Make _ available for import to satisfy static analysis tools
_ = builtins._
