#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.


from fenrirscreenreader.core.i18n import _


class command:
    def __init__(self):
        pass

    def initialize(self, environment):
        self.env = environment

    def shutdown(self):
        pass

    def get_description(self):
        return "No description found"

    def run(self):
        if (
            self.env["input"]["oldCapsLock"]
            == self.env["input"]["newCapsLock"]
        ):
            return
        if self.env["input"]["newCapsLock"]:
            self.env["runtime"]["OutputManager"].present_text(
                _("Capslock on"), interrupt=True
            )
        else:
            self.env["runtime"]["OutputManager"].present_text(
                _("Capslock off"), interrupt=True
            )

    def set_callback(self, callback):
        pass
