#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.


from fenrirscreenreader.core.i18n import _


class command:
    def __init__(self):
        pass

    def initialize(self, environment):
        self.env = environment

    def shutdown(self):
        pass

    def get_description(self):
        return "No description found"

    def load(self):
        return
        print("--------------")
        print("default")
        print("load new", self.env["screen"]["new_application"])
        print("--------------")

    def unload(self):
        return
        print("--------------")
        print("default")
        print("unload old", self.env["screen"]["oldApplication"])
        print("--------------")

    def set_callback(self, callback):
        pass
