#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core import debug


class RemoteDriver:
    def __init__(self):
        self._is_initialized = False

    def initialize(self, environment):
        self.env = environment
        self._is_initialized = True

    def shutdown(self):
        self._is_initialized = False
