#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core import debug
from fenrirscreenreader.core.soundDriver import sound_driver


class driver(sound_driver):
    def __init__(self):
        sound_driver.__init__(self)
