#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

import time

from fenrirscreenreader.core import debug

# used as shared memory between commands
# use this in your own commands
command_buffer = {
    "enableSpeechOnKeypress": False,
    "Marks": {"1": None, "2": None},
    "bookMarks": {},
    "windowArea": {},
}

# used by the command_manager
command_info = {
    # 'curr_command': '',
    "lastCommandExecutionTime": time.time(),
    "lastCommandRequestTime": time.time(),
}
