#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

import time

from fenrirscreenreader.core import debug

output_data = {
    "nextFlush": time.time(),
    "messageText": "",
    "messageOffset": None,
    "cursorOffset": None,
}
