#!/usr/bin/env python3

import datetime
import time

from fenrirscreenreader.core.i18n import _

# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.


class command:
    def __init__(self):
        pass

    def initialize(self, environment):
        self.env = environment
        self.last_time = datetime.datetime.now()
        self.lastDateString = ""
        self.lastTimeString = ""

    def shutdown(self):
        pass

    def get_description(self):
        return "No Description found"

    def run(self):
        self.env["runtime"]["ScreenDriver"].get_session_information()

    def set_callback(self, callback):
        pass
